//+------------------------------------------------------------------+
//|                                                    Fractal.Lines |
//|                                          joelswincer@hotmail.com |
//|                                                                  |
//| Copyright 2009 Joel Swincer. All rights reserved.                |
//|                                                                  |
//| Fractal Lines Indicator                                          |
//|                                                                  |
//|                                                                  |
//|                                                                  |
//|                                                                  |
//| Programmed by                                                    |
//|      Witold Wozniak                                              |
//|                                                                  |
//| Version                                                          |
//|      1.0.0      2009-07-09                                       |
//|                                                                  |
//|                                              contact@mqlsoft.com |
//|                                          http://www.mqlsoft.com/ |
//+------------------------------------------------------------------+
#property copyright "Copyright ? 2009 Joel Swincer; Programmed by Witold Wozniak (www.mqlsoft.com)"
#property link      ""

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 Green
#property indicator_color2 Red

#property indicator_level1 0

#define null EMPTY_VALUE

extern int BarRetrace = 15;
extern int PipsRetrace = 100;
extern int NoBars2Left = 10;

extern bool SessionExtremes = false;
extern int SessionStart = 9;
extern int SessionEnd = 15;

extern int MaxLines = 3;
extern int MaxDuration = 500;

double validUP[];
double validDN[];

int drawBegin = 0;
int buffers = 0;

string shortName;
string PREFIX;

//+------------------------------------------------------------------+

int init() {
    PREFIX = "Fractal.Lines.";
    shortName = "Fractal Lines [" + BarRetrace + ", " + PipsRetrace + ", " + NoBars2Left + "]";
    drawBegin = BarRetrace + NoBars2Left;
    initBuffer(validUP, "Fractal UP", DRAW_ARROW, 233, EMPTY);
    initBuffer(validDN, "Fractal DOWN", DRAW_ARROW, 234, EMPTY);
    
    IndicatorBuffers(buffers);
    IndicatorShortName(shortName);
    return (0);
}

int deinit() {
    for (int cnt = ObjectsTotal() - 1; cnt >= 0; cnt--) {
        string name = ObjectName(cnt);
        if (ObjectType(name) == OBJ_TREND) {
            if (stringStartsWith(name, PREFIX)) {
                ObjectDelete(name);
            }
        }    
    }
    return (0);    
}

int start() {
    if (BarRetrace <= 1) {
        return (-1);
    }
    double tick = getTick();   
    if (Bars <= 1) return (0);
    int countedBars = IndicatorCounted();
    if (countedBars < 0) return (-1);
    if (countedBars > 0) countedBars--;
    int s, i, limit = Bars - countedBars - 1;
    for (s = limit; s >= 0; s--) {
        validUP[s + BarRetrace] = null;
        validDN[s + BarRetrace] = null;
        int index = s + BarRetrace + 1;
        int hour = TimeHour(Time[index]);
        double fractalUP = iFractals(0, 0, MODE_UPPER, index);
        double fractalDN = iFractals(0, 0, MODE_LOWER, index);
        double hA = High[iHighest(0, 0, MODE_HIGH, BarRetrace, s + 1)];
        double lA = Low[iLowest(0, 0, MODE_LOW, BarRetrace, s + 1)];
        double hB = High[iHighest(0, 0, MODE_HIGH, NoBars2Left, index + 1)];
        double lB = Low[iLowest(0, 0, MODE_LOW, NoBars2Left, index + 1)];        
        if (fractalUP != 0 && fractalUP != null) {
            if (hA < High[index] && hB < High[index] && MathRound(High[index] / tick) - MathRound(lA / tick) >= PipsRetrace) {
                if (SessionExtremes) {
                    if (hour >= SessionStart && hour <= SessionEnd) {
                        validUP[index] = High[index];
                    }
                } else {
                    validUP[index] = High[index];
                }
            }
        }
        if (fractalDN != 0 && fractalDN != null) {
            if (lA > Low[index] && lB > Low[index] && MathRound(hA / tick) - MathRound(Low[index] / tick) >= PipsRetrace) {
                if (SessionExtremes) {
                    if (hour >= SessionStart && hour <= SessionEnd) {
                        validDN[index] = Low[index];
                    }
                } else {
                    validDN[index] = Low[index];
                }
            }
        }
    }
    int upCounter = 0, dnCounter = 0;
    for (s = 0; s < Bars; s++) {
        string name = PREFIX + Time[s];
        if (s >= MaxDuration && MaxDuration > EMPTY) {
            if (ObjectFind(name) == 0) {
                ObjectDelete(name);
            }
        } else {
            if (validUP[s] != null) {
                if (upCounter < MaxLines || MaxLines == EMPTY) {
                    if (ObjectFind(name) == EMPTY) {
                        ObjectCreate(name, OBJ_TREND, 0, Time[s], validUP[s], Time[s - BarRetrace], validUP[s]);
                        ObjectSet(name, OBJPROP_RAY, true);
                        ObjectSet(name, OBJPROP_COLOR, Green);
                    }
                } else {
                    if (ObjectFind(name) != EMPTY) {
                        ObjectDelete(name);
                    }
                }
                upCounter++;
            }
            if (validDN[s] != null) {
                if (dnCounter < MaxLines || MaxLines == EMPTY) {
                    if (ObjectFind(name) == EMPTY) {
                        ObjectCreate(name, OBJ_TREND, 0, Time[s], validDN[s], Time[s - BarRetrace], validDN[s]);
                        ObjectSet(name, OBJPROP_RAY, true);
                        ObjectSet(name, OBJPROP_COLOR, Red);
                    }
                } else {
                    if (ObjectFind(name) != EMPTY) {
                        ObjectDelete(name);
                    }
                }
                dnCounter++;
            }
        }
    }
    return (0);
}

void initBuffer(double array[], string label = "", int type = DRAW_NONE, 
        int arrow = 0, int style = EMPTY, int width = EMPTY, color clr = CLR_NONE) {
    SetIndexBuffer(buffers, array);
    SetIndexLabel(buffers, label);
    SetIndexEmptyValue(buffers, null);
    SetIndexDrawBegin(buffers, drawBegin);
    SetIndexShift(buffers, 0);
    if (clr != CLR_NONE) {
        SetIndexStyle(buffers, type, style, width, clr);
    } else {
        SetIndexStyle(buffers, type, style, width);
    }
    SetIndexArrow(buffers, arrow);
    buffers++;
}

double getTick() {
    double tick = MarketInfo(Symbol(), MODE_TICKSIZE);
    if (Digits % 2 == 1) {
        tick *= 10;
    }
    return (tick);
}

bool stringStartsWith(string str, string prefix) {
    return (StringFind(str, prefix) == 0);
}

